package com.care.bms.mqtt;


import cn.hutool.core.date.DateUtil;

import com.alibaba.fastjson.JSON;

import java.util.Map;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.care.bms.service.BmsEventOrderService;
import com.care.common.entity.CareDevice;
import com.care.common.entity.CareEventOrder;
import com.care.common.entity.CareHouse;
import com.care.common.enums.OrderStatusEnum;
import com.care.common.enums.OrderTypeEnum;
import com.care.common.service.*;
import com.care.common.util.CommonConfUtil;
import org.eclipse.paho.client.mqttv3.IMqttMessageListener;
import org.eclipse.paho.client.mqttv3.MqttMessage;


/**
 * 订阅信息监听类
 */
public class MqttMessageListener implements IMqttMessageListener {
    private CareDeviceService careDeviceService;
    private BmsEventOrderService bmsEventOrderService;
    private CareEventOrderService careEventOrderService;
    private CareHouseService careHouseService;
    private CareOrganizationService careOrganizationService;
    private CareStationService careStationService;


    public CareOrganizationService getCareOrganizationService() {
        return careOrganizationService;
    }

    public void setCareOrganizationService(CareOrganizationService careOrganizationService) {
        this.careOrganizationService = careOrganizationService;
    }

    public CareStationService getCareStationService() {
        return careStationService;
    }

    public void setCareStationService(CareStationService careStationService) {
        this.careStationService = careStationService;
    }

    public CareHouseService getCareHouseService() {
        return careHouseService;
    }

    public void setCareHouseService(CareHouseService careHouseService) {
        this.careHouseService = careHouseService;
    }


    public CareEventOrderService getCareEventOrderService() {
        return careEventOrderService;
    }

    public void setCareEventOrderService(CareEventOrderService careEventOrderService) {
        this.careEventOrderService = careEventOrderService;
    }


    public BmsEventOrderService getBmsEventOrderService() {
        return bmsEventOrderService;
    }

    public void setBmsEventOrderService(BmsEventOrderService bmsEventOrderService) {
        this.bmsEventOrderService = bmsEventOrderService;
    }

    public CareDeviceService getCareDeviceService() {
        return careDeviceService;
    }

    public void setCareDeviceService(CareDeviceService careDeviceService) {
        this.careDeviceService = careDeviceService;
    }


    @Override
    public void messageArrived(String topic, MqttMessage mqttMessage) throws Exception {

//        System.out.println(mqttMessage.getId());
//        System.out.println(mqttMessage.getPayload());
//        System.out.println(mqttMessage.getQos());
//        System.out.println(mqttMessage.isRetained());
//        System.out.println(mqttMessage.isDuplicate());

        String mqttMessageStr = mqttMessage.toString();
        System.out.println("mqttMessageStr == " + mqttMessageStr);
        if(mqttMessageStr == null || !mqttMessageStr.startsWith("{")){
            return;
        }
        QueryWrapper<CareDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareDevice::getTopic,topic);
        if(careDeviceService == null){
            return;
        }
        CareDevice careDevice = careDeviceService.getOne(queryWrapper);
        if(careDevice == null) {
            System.out.println("数据库中没找到对应设备");
            return;
        }
        try {
            Map map = JSON.parseObject(mqttMessageStr);
            String type = (String) map.get("type");
            if ("BreathHeartRate".equals(type)) {
                Map msg = (Map) map.get("msg");
                Integer breath = (Integer) msg.get("breath");
                Integer heart = (Integer) msg.get("heart");
                careDevice.setBreathRate(breath);
                careDevice.setHeartRate(heart);
                careDeviceService.updateById(careDevice);
            } else if ("FallDown".equals(type)) {
                saveOrder(careDevice, OrderTypeEnum.DI_EDAO.getValue());
            } else if ("StayTooLong".equals(type)) {
                saveOrder(careDevice, OrderTypeEnum.JIU_ZHI.getValue());
            }
        } catch (Exception e){
            e.printStackTrace();
        }
        System.out.println(topic);
    }

    private void saveOrder(CareDevice careDevice,String orderType ){

        CareHouse careHouse = careHouseService.getById(careDevice.getHouseId());

        CareEventOrder order = new CareEventOrder();
        order.setOrgId(careHouse.getOrgId());
        order.setOrgName(careOrganizationService.getById(careHouse.getOrgId()).getName());
        order.setStationId(careHouse.getStationId());
        order.setStationName(careStationService.getById(careHouse.getStationId()).getName());
        order.setHouseId(careHouse.getId());
        order.setDevId(careDevice.getId());
        order.setDevName(careDevice.getDevName());
        order.setHouseName(careHouse.getName());
        order.setTitle(careHouse.getAddr());
        order.setOrderType(orderType);
        order.setStatus(OrderStatusEnum.TODO.getValue());
        order.setCreateTime(DateUtil.date());
        order.setModifyTime(DateUtil.date());
        careEventOrderService.saveOrder(order);

    }

}
