package com.care.bms.service;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.care.common.entity.CareOrganization;
import com.care.common.entity.CareSysUser;
import com.care.common.enums.UserRoleEnum;
import com.care.common.exception.BDException;
import com.care.common.service.CareOrganizationService;
import com.care.common.service.CareSysUserService;
import com.care.common.vo.UserLogindConvertVO;
import com.care.common.vo.sysuser.CareSysUserVO;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 安全管家表(CareSysUser)表服务实现类
 *
 * @author makejava
 * @since 2021-05-21 00:08:38
 */
@Service
public class BmsSysUserService{

    @Resource
    private CareOrganizationService careOrganizationService;
    @Resource
    private CareSysUserService careSysUserService;

    public UserLogindConvertVO getLoginUser(String orgCode, String phone,String password) {
        UserLogindConvertVO userLogindConvertVO = null;
        QueryWrapper<CareOrganization>  organizationQueryWrapper = new QueryWrapper<>();
        organizationQueryWrapper.lambda().eq(CareOrganization::getOrgCode,orgCode);
        CareOrganization org = this.careOrganizationService.getOne(organizationQueryWrapper);
        if (org == null){
            return null;
        }
        QueryWrapper<CareSysUser>  userQueryWrapper = new QueryWrapper<>();
        userQueryWrapper.lambda().eq(CareSysUser::getOrgId,org.getId())
                                .eq(CareSysUser::getPhone,phone);
        CareSysUser user = this.careSysUserService.getOne(userQueryWrapper);
        if (user != null){
            userLogindConvertVO = new UserLogindConvertVO();
            BeanUtil.copyProperties(user,userLogindConvertVO);
            userLogindConvertVO.setOrgName(org.getName());
            String md5Password = DigestUtils.md5Hex(password);
            if (!md5Password.equalsIgnoreCase(user.getPassword())) {
                throw new BDException("用户名或密码错误");
            }
        }
        return userLogindConvertVO;
    }

    /**
     * 查询某服务站下的 安全管家列表
     * @param stationId
     * @return
     */
    public List<CareSysUserVO> queryKeeperListByStationId(Long stationId){
        List<CareSysUserVO> datas = new ArrayList<>();
        QueryWrapper<CareSysUser>  userQueryWrapper = new QueryWrapper<>();
        userQueryWrapper.lambda().eq(CareSysUser::getStationId,stationId)
                .eq(CareSysUser::getRole, UserRoleEnum.CHANMB.getValue())
                .orderByAsc(CareSysUser::getName);
        List<CareSysUser> users = this.careSysUserService.list(userQueryWrapper);
        if (users != null){
            users.forEach(item -> {
                CareSysUserVO vo = new CareSysUserVO();
                BeanUtil.copyProperties(item,vo);
                datas.add(vo);
            });
        }
        return datas;
    }
}