package com.care.bms.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.HexUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.care.bms.mqtt.MqttTool;
import com.care.common.entity.CareDevice;
import com.care.common.entity.CareHouse;
import com.care.common.entity.CareStation;
import com.care.common.enums.DeviceStatusEnum;
import com.care.common.exception.BDException;
import com.care.common.service.*;
import com.care.common.vo.PageReqVO;
import com.care.common.vo.UserLogindConvertVO;
import com.care.common.vo.device.DeviceVO;
import lombok.Synchronized;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;

/**
 * @Author: lilt
 * @Date: 2021/7/14
 * @Desc:
 */
@Service
public class DeviceService {
    @Resource
    private CareHouseService careHouseService;
    @Resource
    private CareStationService careStationService;
    @Resource
    private CareDeviceService careDeviceService;

    @Resource
    private BmsEventOrderService bmsEventOrderService;
    @Resource
    private CareEventOrderService careEventOrderService;
    @Resource
    private CareOrganizationService careOrganizationService;
    /**
     * 删除设备
     * @param id
     * @return
     */
    public boolean deleteById(Long id){
        MqttTool.removeOne(this.careDeviceService.getById(id));
        return this.careHouseService.removeById(id);
    }

    public DeviceVO getDevById(Long id){
        CareDevice dev = this.careDeviceService.getById(id);
        if (dev!=null){
            DeviceVO vo = new DeviceVO();
            BeanUtil.copyProperties(dev,vo);
            vo.setStatusName(DeviceStatusEnum.getCodeToName(vo.getStatus()));
            CareStation station = this.careStationService.getById(vo.getStationId());
            if (station!=null){
                vo.setStationName(station.getShortName());
            }
            CareHouse house = this.careHouseService.getById(vo.getHouseId());
            if (house!=null){
                vo.setHouseAddr(house.getAddr());
            }
            return vo;
        }
        return null;
    }

    /**
     * 保存设备
     * @param vo
     */
    @Transactional(rollbackFor = Exception.class)
    @Synchronized
    public void createDevice(DeviceVO vo,UserLogindConvertVO loginUser){
        //生成登陆账号
        if (StrUtil.isEmpty(vo.getDevCode())){
            throw new BDException("设备编号不能为空");
        }else {
            QueryWrapper<CareDevice> queryWrapper = new QueryWrapper();
            queryWrapper.lambda().eq(CareDevice::getDevCode,vo.getDevCode());
            if (this.careDeviceService.count(queryWrapper)>0){
                throw new BDException("设备编号已经存在！");
            }
        }
        //保存设备
        CareDevice device = new CareDevice();
        BeanUtil.copyProperties(vo,device);
        device.setStatus(DeviceStatusEnum.ONLINE.getValue());
        device.setCreateTime(new Date());
        device.setOrgId(loginUser.getOrgId());
        device.setTopic("5JPD/monitor/"+device.getDevCode()+"/event");
        Long time = System.currentTimeMillis();
        device.setClientId(HexUtil.toHex(time));
        this.careDeviceService.save(device);
        //保存完后，启动通道
        MqttTool.addOne(device, careDeviceService, bmsEventOrderService, careEventOrderService, careHouseService, careOrganizationService, careStationService);

    }

    /**
     * 保存设备
     * @param vo
     */
    @Transactional(rollbackFor = Exception.class)
    public void updateDevice(DeviceVO vo){
        //保存设备
        CareDevice device = this.careDeviceService.getById(vo.getId());
        MqttTool.removeOne(device);
        BeanUtil.copyProperties(vo,device);
        device.setTopic("5JPD/monitor/"+device.getDevCode()+"/event");
        device.setModifyTime(new Date());
        this.careDeviceService.updateById(device);
        //保存完后，启动通道
        MqttTool.addOne(device, careDeviceService, bmsEventOrderService, careEventOrderService, careHouseService, careOrganizationService, careStationService);
    }
    /**
     * 列表查询
     * @param loginUser
     * @param pageReqVo
     * @return
     */
    public IPage<DeviceVO> listDevice(String addr, Long  stationId, String status, String devCode,UserLogindConvertVO loginUser, PageReqVO pageReqVo) {
        return this.careDeviceService.queryList(pageReqVo,stationId,status,addr,devCode,loginUser);
    }
}
